@@ -14,6 +14,7 @@ const saveCodeMsg = {
   'warning': 'The "save code now" request has been put in pending state and may be accepted for processing after manual review.',
   'rejected': 'The "save code now" request has been rejected because the provided origin url is blacklisted.',
   'rateLimit': 'The rate limit for "save code now" requests has been reached. Please try again later.',
+  'not-found': 'The provided url does not exist',
   'unknownError': 'An unexpected error happened when submitting the "save code now request',
   'csrfError': 'CSRF Failed: Referrer checking failed - no Referrer.'
 };
@@ -43,14 +44,16 @@ function stubSaveRequest({
   originUrl,
   saveTaskStatus,
   responseStatus = 200,
+  // For error code with the error message in the 'reason' key response
   errorMessage = '',
   saveRequestDate = new Date(),
-  visitDate = new Date()
+  visitDate = new Date(),
+  visitStatus = null
 } = {}) {
   let response;
   if (responseStatus !== 200 && errorMessage) {
     response = {
-      'detail': errorMessage
+      'reason': errorMessage
     };
   } else {
     response = genOriginSaveResponse({visitType: visitType,
@@ -58,21 +61,24 @@ function stubSaveRequest({
                                       originUrl: originUrl,
                                       saveRequestDate: saveRequestDate,
                                       saveTaskStatus: saveTaskStatus,
-                                      visitDate: visitDate});
+                                      visitDate: visitDate,
+                                      visitStatus: visitStatus
+    });
   }
   cy.intercept('POST', requestUrl, {body: response, statusCode: responseStatus})
     .as('saveRequest');
 }
 
 // Mocks API response : /save/(:visit_type)/(:origin_url)
-// visit_type : {'git', 'hg', 'svn'}
+// visit_type : {'git', 'hg', 'svn', ...}
 function genOriginSaveResponse({
   visitType = 'git',
   saveRequestStatus,
   originUrl,
   saveRequestDate = new Date(),
   saveTaskStatus,
-  visitDate = new Date()
+  visitDate = new Date(),
+  visitStatus
 } = {}) {
   return {
     'visit_type': visitType,
@@ -81,7 +87,8 @@ function genOriginSaveResponse({
     'id': 1,
     'save_request_date': saveRequestDate ? saveRequestDate.toISOString() : null,
     'save_task_status': saveTaskStatus,
-    'visit_date': visitDate ? visitDate.toISOString() : null
+    'visit_date': visitDate ? visitDate.toISOString() : null,
+    'visit_status': visitStatus
   };
 };
 
@@ -89,7 +96,7 @@ describe('Origin Save Tests', function() {
   before(function() {
     url = this.Urls.origin_save();
     origin = this.origin[0];
-    this.originSaveUrl = this.Urls.origin_save_request(origin.type, origin.url);
+    this.originSaveUrl = this.Urls.api_1_save_origin(origin.type, origin.url);
   });
 
   beforeEach(function() {
@@ -98,6 +105,27 @@ describe('Origin Save Tests', function() {
     cy.visit(url);
   });
 
+  it('should format appropriately values depending on their type', function() {
+    let inputValues = [ // null values stay null
+      {type: 'json', value: null, expectedValue: null},
+      {type: 'date', value: null, expectedValue: null},
+      {type: 'raw', value: null, expectedValue: null},
+      {type: 'duration', value: null, expectedValue: null},
+      // non null values formatted depending on their type
+      {type: 'json', value: '{}', expectedValue: '"{}"'},
+      {type: 'date', value: '04/04/2021 01:00:00', expectedValue: '4/4/2021, 1:00:00 AM'},
+      {type: 'raw', value: 'value-for-identity', expectedValue: 'value-for-identity'},
+      {type: 'duration', value: '10', expectedValue: '10 seconds'},
+      {type: 'duration', value: 100, expectedValue: '100 seconds'}
+    ];
+    cy.window().then(win => {
+      inputValues.forEach(function(input, index, array) {
+        let actualValue = win.swh.save.formatValuePerType(input.type, input.value);
+        assert.equal(actualValue, input.expectedValue);
+      });
+    });
+  });
+
   it('should display accepted message when accepted', function() {
     stubSaveRequest({requestUrl: this.originSaveUrl,
                      saveRequestStatus: 'accepted',
@@ -113,7 +141,7 @@ describe('Origin Save Tests', function() {
 
   it('should validate gitlab subproject url', function() {
     const gitlabSubProjectUrl = 'https://gitlab.com/user/project/sub/';
-    const originSaveUrl = this.Urls.origin_save_request('git', gitlabSubProjectUrl);
+    const originSaveUrl = this.Urls.api_1_save_origin('git', gitlabSubProjectUrl);
 
     stubSaveRequest({requestUrl: originSaveUrl,
                      saveRequestStatus: 'accepted',
@@ -129,7 +157,7 @@ describe('Origin Save Tests', function() {
 
   it('should validate project url with _ in username', function() {
     const gitlabSubProjectUrl = 'https://gitlab.com/user_name/project.git';
-    const originSaveUrl = this.Urls.origin_save_request('git', gitlabSubProjectUrl);
+    const originSaveUrl = this.Urls.api_1_save_origin('git', gitlabSubProjectUrl);
 
     stubSaveRequest({requestUrl: originSaveUrl,
                      saveRequestStatus: 'accepted',
@@ -156,6 +184,19 @@ describe('Origin Save Tests', function() {
     });
   });
 
+  it('should show error when the origin does not exist (status: 400)', function() {
+    stubSaveRequest({requestUrl: this.originSaveUrl,
+                     originUrl: origin.url,
+                     responseStatus: 400,
+                     errorMessage: saveCodeMsg['not-found']});
+
+    makeOriginSaveRequest(origin.type, origin.url);
+
+    cy.wait('@saveRequest').then(() => {
+      checkAlertVisible('danger', saveCodeMsg['not-found']);
+    });
+  });
+
   it('should show error when csrf validation failed (status: 403)', function() {
     stubSaveRequest({requestUrl: this.originSaveUrl,
                      saveRequestStatus: 'rejected',
@@ -249,7 +290,8 @@ describe('Origin Save Tests', function() {
       saveRequestStatus: 'accepted',
       originUrl: originUrl,
       saveTaskStatus: 'succeeded',
-      visitDate: null
+      visitDate: null,
+      visitStatus: 'full'
     });
     const saveRequestsListData = {
       'recordsTotal': 1,
@@ -330,11 +372,12 @@ describe('Origin Save Tests', function() {
     const badVisitType = 'hg';
     const goodVisitType = 'git';
     cy.intercept('/save/requests/list/**', {fixture: 'origin-save'});
-    stubSaveRequest({requestUrl: this.Urls.origin_save_request(badVisitType, originUrl),
+    stubSaveRequest({requestUrl: this.Urls.api_1_save_origin(badVisitType, originUrl),
                      visitType: badVisitType,
                      saveRequestStatus: 'accepted',
                      originUrl: originUrl,
                      saveTaskStatus: 'failed',
+                     visitStatus: 'failed',
                      responseStatus: 200,
                      errorMessage: saveCodeMsg['accepted']});
 
@@ -356,4 +399,20 @@ describe('Origin Save Tests', function() {
     });
   });
 
+  it('should create save request for authenticated user', function() {
+    cy.adminLogin();
+    cy.visit(url);
+    const originUrl = 'https://git.example.org/account/repo';
+    stubSaveRequest({requestUrl: this.Urls.api_1_save_origin('git', originUrl),
+                     saveRequestStatus: 'accepted',
+                     originUrl: origin.url,
+                     saveTaskStatus: 'not yet scheduled'});
+
+    makeOriginSaveRequest('git', originUrl);
+
+    cy.wait('@saveRequest').then(() => {
+      checkAlertVisible('success', saveCodeMsg['success']);
+    });
+  });
+
 });
