@@ -38,7 +38,11 @@ from swh.web.common.models import (
     SaveUnauthorizedOrigin,
 )
 from swh.web.common.origin_visits import get_origin_visits
-from swh.web.common.typing import OriginInfo
+from swh.web.common.typing import (
+    OriginExistenceCheckInfo,
+    OriginInfo,
+    SaveOriginRequestInfo,
+)
 from swh.web.common.utils import SWH_WEB_METRICS_REGISTRY, parse_iso8601_date_to_utc
 
 scheduler = config.scheduler()
@@ -68,7 +72,7 @@ def get_origin_save_unauthorized_urls() -> List[str]:
     return [origin.url for origin in SaveUnauthorizedOrigin.objects.all()]
 
 
-def can_save_origin(origin_url: str) -> str:
+def can_save_origin(origin_url: str, bypass_pending_review: bool = False) -> str:
     """
     Check if a software origin can be saved into the archive.
 
@@ -95,8 +99,14 @@ def can_save_origin(origin_url: str) -> str:
         if origin_url.startswith(url_prefix):
             return SAVE_REQUEST_ACCEPTED
 
-    # otherwise, the origin url needs to be manually verified
-    return SAVE_REQUEST_PENDING
+    # otherwise, the origin url needs to be manually verified if the user
+    # that submitted it does not have special permission
+    if bypass_pending_review:
+        # mark the origin URL as trusted in that case
+        SaveAuthorizedOrigin.objects.get_or_create(url=origin_url)
+        return SAVE_REQUEST_ACCEPTED
+    else:
+        return SAVE_REQUEST_PENDING
 
 
 # map visit type to scheduler task
@@ -156,9 +166,49 @@ def _check_origin_url_valid(origin_url: str) -> None:
         )
 
 
+def origin_exists(origin_url: str) -> OriginExistenceCheckInfo:
+    """Check the origin url for existence. If it exists, extract some more useful
+    information on the origin.
+
+    """
+    resp = requests.head(origin_url)
+    exists = resp.ok
+    content_length: Optional[int] = None
+    last_modified: Optional[str] = None
+    if exists:
+        size_ = resp.headers.get("Content-Length")
+        content_length = int(size_) if size_ else None
+        last_modified = resp.headers.get("Last-Modified")
+
+    return OriginExistenceCheckInfo(
+        origin_url=origin_url,
+        exists=exists,
+        last_modified=last_modified,
+        content_length=content_length,
+    )
+
+
+def _check_origin_exists(origin_url: str) -> None:
+    """Ensure the origin exists, if not raise an explicit message."""
+    check = origin_exists(origin_url)
+    if not check["exists"]:
+        raise BadInputExc(
+            f"The provided origin url ({escape(origin_url)}) does not exist!"
+        )
+
+
 def _get_visit_info_for_save_request(
     save_request: SaveOriginRequest,
 ) -> Tuple[Optional[datetime], Optional[str]]:
+    """Retrieve visit information out of a save request
+
+    Args:
+        save_request: Input save origin request to retrieve information for.
+
+    Returns:
+        Tuple of (visit date, optional visit status) for such save request origin
+
+    """
     visit_date = None
     visit_status = None
     time_now = datetime.now(tz=timezone.utc)
@@ -175,7 +225,7 @@ def _get_visit_info_for_save_request(
             if i != len(visit_dates):
                 visit_date = visit_dates[i]
                 visit_status = origin_visits[i]["status"]
-                if origin_visits[i]["status"] not in ("full", "partial", "not_found"):
+                if visit_status not in ("full", "partial", "not_found"):
                     visit_date = None
         except Exception as exc:
             sentry_sdk.capture_exception(exc)
@@ -185,12 +235,24 @@ def _get_visit_info_for_save_request(
 def _check_visit_update_status(
     save_request: SaveOriginRequest, save_task_status: str
 ) -> Tuple[Optional[datetime], str]:
+    """Given a save request and a save task status, determine whether a save request was
+    successful or failed.
+
+    Args:
+        save_request: Input save origin request to retrieve information for.
+
+    Returns:
+        Tuple of (optional visit date, save task status) for such save request origin
+
+    """
     visit_date, visit_status = _get_visit_info_for_save_request(save_request)
     save_request.visit_date = visit_date
-    # visit has been performed, mark the saving task as succeed
-    if visit_date and visit_status is not None:
+    save_request.visit_status = visit_status
+    if visit_date and visit_status in ("full", "partial"):
+        # visit has been performed, mark the saving task as succeeded
         save_task_status = SAVE_TASK_SUCCEEDED
     elif visit_status in ("created", "ongoing"):
+        # visit is currently running
         save_task_status = SAVE_TASK_RUNNING
     elif visit_status in ("not_found", "failed"):
         save_task_status = SAVE_TASK_FAILED
@@ -204,18 +266,31 @@ def _check_visit_update_status(
     return visit_date, save_task_status
 
 
-def _save_request_dict(
+def _update_save_request_info(
     save_request: SaveOriginRequest,
     task: Optional[Dict[str, Any]] = None,
     task_run: Optional[Dict[str, Any]] = None,
-) -> Dict[str, Any]:
+) -> SaveOriginRequestInfo:
+    """Update save request information out of task and task_run information.
+
+    Args:
+        save_request: Save request
+        task: Associated scheduler task information about the save request
+        task_run: Most recent run occurrence of the associated task
+
+    Returns:
+        Summary of the save request information updated.
+
+    """
     must_save = False
     visit_date = save_request.visit_date
+
     # save task still in scheduler db
     if task:
         save_task_status = _save_task_status[task["status"]]
         if task_run:
             save_task_status = _save_task_run_status[task_run["status"]]
+
         # Consider request from which a visit date has already been found
         # as succeeded to avoid retrieving it again
         if save_task_status == SAVE_TASK_SCHEDULED and visit_date:
@@ -226,6 +301,7 @@ def _save_request_dict(
         ):
             visit_date, visit_status = _get_visit_info_for_save_request(save_request)
             save_request.visit_date = visit_date
+            save_request.visit_status = visit_status
             if visit_status in ("failed", "not_found"):
                 save_task_status = SAVE_TASK_FAILED
             must_save = True
@@ -258,18 +334,12 @@ def _save_request_dict(
     if must_save:
         save_request.save()
 
-    return {
-        "id": save_request.id,
-        "visit_type": save_request.visit_type,
-        "origin_url": save_request.origin_url,
-        "save_request_date": save_request.request_date.isoformat(),
-        "save_request_status": save_request.status,
-        "save_task_status": save_request.loading_task_status,
-        "visit_date": visit_date.isoformat() if visit_date else None,
-    }
+    return save_request.to_dict()
 
 
-def create_save_origin_request(visit_type: str, origin_url: str) -> Dict[str, Any]:
+def create_save_origin_request(
+    visit_type: str, origin_url: str, bypass_pending_review: bool = False
+) -> SaveOriginRequestInfo:
     """
     Create a loading task to save a software origin into the archive.
 
@@ -285,12 +355,11 @@ def create_save_origin_request(visit_type: str, origin_url: str) -> Dict[str, An
     database to keep track of them.
 
     Args:
-        visit_type (str): the type of visit to perform (currently only
-            ``git`` but ``svn`` and ``hg`` will soon be available)
-        origin_url (str): the url of the origin to save
+        visit_type: the type of visit to perform (e.g git, hg, svn, ...)
+        origin_url: the url of the origin to save
 
     Raises:
-        BadInputExc: the visit type or origin url is invalid
+        BadInputExc: the visit type or origin url is invalid or inexistent
         ForbiddenExc: the provided origin url is blacklisted
 
     Returns:
@@ -309,7 +378,8 @@ def create_save_origin_request(visit_type: str, origin_url: str) -> Dict[str, An
     """
     _check_visit_type_savable(visit_type)
     _check_origin_url_valid(origin_url)
-    save_request_status = can_save_origin(origin_url)
+    # if all checks passed so far, we can try and save the origin
+    save_request_status = can_save_origin(origin_url, bypass_pending_review)
     task = None
 
     # if the origin save request is accepted, create a scheduler
@@ -346,8 +416,8 @@ def create_save_origin_request(visit_type: str, origin_url: str) -> Dict[str, An
                 task = tasks[0] if tasks else None
                 task_runs = scheduler.get_task_runs([sor.loading_task_id])
                 task_run = task_runs[0] if task_runs else None
-                save_request = _save_request_dict(sor, task, task_run)
-                task_status = save_request["save_task_status"]
+                save_request_info = _update_save_request_info(sor, task, task_run)
+                task_status = save_request_info["save_task_status"]
                 # create a new scheduler task only if the previous one has been
                 # already executed
                 if (
@@ -405,22 +475,22 @@ def create_save_origin_request(visit_type: str, origin_url: str) -> Dict[str, An
         )
 
     assert sor is not None
-    return _save_request_dict(sor, task)
+    return _update_save_request_info(sor, task)
 
 
-def get_save_origin_requests_from_queryset(
+def update_save_origin_requests_from_queryset(
     requests_queryset: QuerySet,
-) -> List[Dict[str, Any]]:
-    """
-    Get all save requests from a SaveOriginRequest queryset.
+) -> List[SaveOriginRequestInfo]:
+    """Update all save requests from a SaveOriginRequest queryset, update their status in db
+    and return the list of impacted save_requests.
 
     Args:
-        requests_queryset (django.db.models.QuerySet): input
-            SaveOriginRequest queryset
+        requests_queryset: input SaveOriginRequest queryset
 
     Returns:
-        list: A list of save origin requests dict as described in
+        list: A list of save origin request info dicts as described in
         :func:`swh.web.common.origin_save.create_save_origin_request`
+
     """
     task_ids = []
     for sor in requests_queryset:
@@ -432,14 +502,45 @@ def get_save_origin_requests_from_queryset(
         task_runs = scheduler.get_task_runs(tasks)
         task_runs = {task_run["task"]: task_run for task_run in task_runs}
         for sor in requests_queryset:
-            sr_dict = _save_request_dict(
-                sor, tasks.get(sor.loading_task_id), task_runs.get(sor.loading_task_id)
+            sr_dict = _update_save_request_info(
+                sor, tasks.get(sor.loading_task_id), task_runs.get(sor.loading_task_id),
             )
             save_requests.append(sr_dict)
     return save_requests
 
 
-def get_save_origin_requests(visit_type: str, origin_url: str) -> List[Dict[str, Any]]:
+def refresh_save_origin_request_statuses() -> List[SaveOriginRequestInfo]:
+    """Refresh non-terminal save origin requests (SOR) in the backend.
+
+    Non-terminal SOR are requests whose status is **accepted** and their task status are
+    either **created**, **not yet scheduled**, **scheduled** or **running**.
+
+    This shall compute this list of SOR, checks their status in the scheduler and
+    optionally elasticsearch for their current status. Then update those in db.
+
+    Finally, this returns the refreshed information on those SOR.
+
+    """
+    non_terminal_statuses = (
+        SAVE_TASK_NOT_CREATED,
+        SAVE_TASK_NOT_YET_SCHEDULED,
+        SAVE_TASK_RUNNING,
+        SAVE_TASK_SCHEDULED,
+    )
+    save_requests = SaveOriginRequest.objects.filter(
+        status=SAVE_REQUEST_ACCEPTED, loading_task_status__in=non_terminal_statuses
+    )
+    # update save request statuses
+    return (
+        update_save_origin_requests_from_queryset(save_requests)
+        if save_requests.count() > 0
+        else []
+    )
+
+
+def get_save_origin_requests(
+    visit_type: str, origin_url: str
+) -> List[SaveOriginRequestInfo]:
     """
     Get all save requests for a given software origin.
 
@@ -466,7 +567,7 @@ def get_save_origin_requests(visit_type: str, origin_url: str) -> List[Dict[str,
             ("No save requests found for visit of type " "%s on origin with url %s.")
             % (visit_type, origin_url)
         )
-    return get_save_origin_requests_from_queryset(sors)
+    return update_save_origin_requests_from_queryset(sors)
 
 
 def get_save_origin_task_info(
@@ -493,6 +594,7 @@ def get_save_origin_task_info(
             - **scheduled**: loading task scheduling date
             - **ended**: loading task termination date
             - **status**: loading task execution status
+            - **visit_status**: Actual visit status
 
         Depending on the availability of the task logs in the elasticsearch
         cluster of Software Heritage, the returned dictionary may also
@@ -522,6 +624,8 @@ def get_save_origin_task_info(
     task_run["id"] = task_run["task"]
     del task_run["task"]
     del task_run["metadata"]
+    # Enrich the task run with the loading visit status
+    task_run["visit_status"] = save_request.visit_status
 
     es_workers_index_url = config.get_config()["es_workers_index_url"]
     if not es_workers_index_url:
@@ -681,7 +785,7 @@ def compute_save_requests_metrics() -> None:
     for sor in SaveOriginRequest.objects.all():
         if sor.status == SAVE_REQUEST_ACCEPTED:
             _accepted_save_requests_gauge.labels(
-                load_task_status=sor.loading_task_status, visit_type=sor.visit_type
+                load_task_status=sor.loading_task_status, visit_type=sor.visit_type,
             ).inc()
 
         _submitted_save_requests_gauge.labels(
@@ -695,5 +799,5 @@ def compute_save_requests_metrics() -> None:
         ):
             delay = sor.visit_date.timestamp() - sor.request_date.timestamp()
             _accepted_save_requests_delay_gauge.labels(
-                load_task_status=sor.loading_task_status, visit_type=sor.visit_type
+                load_task_status=sor.loading_task_status, visit_type=sor.visit_type,
             ).inc(delay)
