@@ -1,4 +1,4 @@
-# Copyright (C) 2018-2019  The Software Heritage developers
+# Copyright (C) 2018-2021  The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU Affero General Public License version 3, or any later version
 # See top-level LICENSE file for more information
@@ -7,8 +7,10 @@ from datetime import datetime, timedelta
 
 import pytest
 
+from django.core.exceptions import ObjectDoesNotExist
 from django.utils import timezone
 
+from swh.web.auth.utils import SWH_AMBASSADOR_PERMISSION
 from swh.web.common.models import (
     SAVE_REQUEST_ACCEPTED,
     SAVE_REQUEST_PENDING,
@@ -18,17 +20,28 @@ from swh.web.common.models import (
     SAVE_TASK_NOT_YET_SCHEDULED,
     SAVE_TASK_SCHEDULED,
     SAVE_TASK_SUCCEEDED,
+    VISIT_STATUS_FAILED,
+    VISIT_STATUS_FULL,
+    SaveAuthorizedOrigin,
     SaveOriginRequest,
     SaveUnauthorizedOrigin,
 )
+from swh.web.common.typing import OriginExistenceCheckInfo
 from swh.web.common.utils import reverse
-from swh.web.tests.utils import check_api_get_responses, check_api_post_responses
+from swh.web.settings.tests import save_origin_rate_post
+from swh.web.tests.utils import (
+    check_api_get_responses,
+    check_api_post_response,
+    check_api_post_responses,
+)
 
 pytestmark = pytest.mark.django_db
 
 
 @pytest.fixture(autouse=True)
 def populated_db():
+    SaveAuthorizedOrigin.objects.create(url="https://github.com/"),
+    SaveAuthorizedOrigin.objects.create(url="https://gitlab.com/"),
     SaveUnauthorizedOrigin.objects.create(url="https://github.com/user/illegal_repo")
     SaveUnauthorizedOrigin.objects.create(url="https://gitlab.com/user_to_exclude")
 
@@ -63,6 +76,11 @@ def check_created_save_request_status(
 ):
 
     mock_scheduler = mocker.patch("swh.web.common.origin_save.scheduler")
+    mock_origin_exists = mocker.patch("swh.web.common.origin_save.origin_exists")
+    mock_origin_exists.return_value = OriginExistenceCheckInfo(
+        origin_url=origin_url, exists=True, last_modified=None, content_length=None
+    )
+
     if scheduler_task_status is None:
         mock_scheduler.get_tasks.return_value = []
     else:
@@ -109,7 +127,7 @@ def check_created_save_request_status(
     )
 
     mock_visit_date = mocker.patch(
-        ("swh.web.common.origin_save." "_get_visit_info_for_save_request")
+        ("swh.web.common.origin_save._get_visit_info_for_save_request")
     )
     mock_visit_date.return_value = (visit_date, None)
 
@@ -130,6 +148,7 @@ def check_save_request_status(
     scheduler_task_status="next_run_not_scheduled",
     scheduler_task_run_status=None,
     visit_date=None,
+    visit_status=None,
 ):
     mock_scheduler = mocker.patch("swh.web.common.origin_save.scheduler")
     mock_scheduler.get_tasks.return_value = [
@@ -164,14 +183,15 @@ def check_save_request_status(
     )
 
     mock_visit_date = mocker.patch(
-        ("swh.web.common.origin_save." "_get_visit_info_for_save_request")
+        ("swh.web.common.origin_save._get_visit_info_for_save_request")
     )
-    mock_visit_date.return_value = (visit_date, None)
+    mock_visit_date.return_value = (visit_date, visit_status)
     response = check_api_get_responses(api_client, url, status_code=200)
     save_request_data = response.data[0]
 
     assert save_request_data["save_request_status"] == expected_request_status
     assert save_request_data["save_task_status"] == expected_task_status
+    assert save_request_data["visit_status"] == visit_status
 
     # Check that save task status is still available when
     # the scheduler task has been archived
@@ -179,6 +199,7 @@ def check_save_request_status(
     response = check_api_get_responses(api_client, url, status_code=200)
     save_request_data = response.data[0]
     assert save_request_data["save_task_status"] == expected_task_status
+    assert save_request_data["visit_status"] == visit_status
 
 
 def test_save_request_rejected(api_client, mocker):
@@ -251,6 +272,7 @@ def test_save_request_succeed(api_client, mocker):
         scheduler_task_status="completed",
         scheduler_task_run_status="eventful",
         visit_date=visit_date,
+        visit_status=VISIT_STATUS_FULL,
     )
 
 
@@ -280,6 +302,7 @@ def test_save_request_failed(api_client, mocker):
         expected_task_status=SAVE_TASK_FAILED,
         scheduler_task_status="disabled",
         scheduler_task_run_status="failed",
+        visit_status=VISIT_STATUS_FAILED,
     )
 
 
@@ -360,7 +383,96 @@ def test_get_save_requests_unknown_origin(api_client):
     assert response.data == {
         "exception": "NotFoundExc",
         "reason": (
-            "No save requests found for visit of type " "git on origin with url %s."
+            "No save requests found for visit of type git on origin with url %s."
         )
         % unknown_origin_url,
     }
+
+
+_visit_type = "git"
+_origin_url = "https://github.com/python/cpython"
+
+
+def test_save_requests_rate_limit(api_client, mocker):
+    create_save_origin_request = mocker.patch(
+        "swh.web.api.views.origin_save.create_save_origin_request"
+    )
+
+    def _save_request_dict(*args, **kwargs):
+        return {
+            "id": 1,
+            "visit_type": _visit_type,
+            "origin_url": _origin_url,
+            "save_request_date": datetime.now().isoformat(),
+            "save_request_status": SAVE_REQUEST_ACCEPTED,
+            "save_task_status": SAVE_TASK_NOT_YET_SCHEDULED,
+            "visit_date": None,
+            "visit_status": None,
+        }
+
+    create_save_origin_request.side_effect = _save_request_dict
+
+    url = reverse(
+        "api-1-save-origin",
+        url_args={"visit_type": _visit_type, "origin_url": _origin_url},
+    )
+
+    for _ in range(save_origin_rate_post):
+        check_api_post_response(api_client, url, status_code=200)
+
+    check_api_post_response(api_client, url, status_code=429)
+
+
+def test_save_request_form_server_error(api_client, mocker):
+    create_save_origin_request = mocker.patch(
+        "swh.web.api.views.origin_save.create_save_origin_request"
+    )
+    create_save_origin_request.side_effect = Exception("Server error")
+
+    url = reverse(
+        "api-1-save-origin",
+        url_args={"visit_type": _visit_type, "origin_url": _origin_url},
+    )
+
+    check_api_post_responses(api_client, url, status_code=500)
+
+
+@pytest.fixture
+def origin_to_review():
+    return "https://git.example.org/user/project"
+
+
+def test_create_save_request_pending_review_anonymous_user(
+    api_client, origin_to_review, requests_mock
+):
+
+    url = reverse(
+        "api-1-save-origin",
+        url_args={"visit_type": "git", "origin_url": origin_to_review},
+    )
+
+    response = check_api_post_responses(api_client, url, status_code=200)
+
+    assert response.data["save_request_status"] == SAVE_REQUEST_PENDING
+
+    with pytest.raises(ObjectDoesNotExist):
+        SaveAuthorizedOrigin.objects.get(url=origin_to_review)
+
+
+def test_create_save_request_accepted_ambassador_user(
+    api_client, origin_to_review, requests_mock, keycloak_oidc, mocker
+):
+
+    keycloak_oidc.realm_permissions += [SWH_AMBASSADOR_PERMISSION]
+    oidc_profile = keycloak_oidc.login()
+    api_client.credentials(HTTP_AUTHORIZATION=f"Bearer {oidc_profile['refresh_token']}")
+
+    check_created_save_request_status(
+        api_client,
+        mocker,
+        origin_to_review,
+        expected_request_status=SAVE_REQUEST_ACCEPTED,
+        expected_task_status=SAVE_TASK_NOT_YET_SCHEDULED,
+    )
+
+    assert SaveAuthorizedOrigin.objects.get(url=origin_to_review)
