@@ -1,4 +1,4 @@
-# Copyright (C) 2019-2020  The Software Heritage developers
+# Copyright (C) 2019-2021  The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU Affero General Public License version 3, or any later version
 # See top-level LICENSE file for more information
@@ -6,23 +6,34 @@
 from datetime import datetime, timedelta, timezone
 from functools import partial
 import re
+from typing import Optional
 
 import pytest
 import requests
 
 from swh.core.pytest_plugin import get_response_cb
+from swh.web.common.exc import BadInputExc
 from swh.web.common.models import (
     SAVE_REQUEST_ACCEPTED,
     SAVE_TASK_FAILED,
     SAVE_TASK_RUNNING,
+    SAVE_TASK_SCHEDULED,
     SAVE_TASK_SUCCEEDED,
+    VISIT_STATUS_FULL,
     SaveOriginRequest,
 )
 from swh.web.common.origin_save import (
+    _check_origin_exists,
     get_save_origin_requests,
     get_save_origin_task_info,
+    origin_exists,
+    refresh_save_origin_request_statuses,
+)
+from swh.web.common.typing import (
+    OriginExistenceCheckInfo,
+    OriginVisitInfo,
+    SaveOriginRequestInfo,
 )
-from swh.web.common.typing import OriginVisitInfo
 from swh.web.config import get_config
 
 _es_url = "http://esnode1.internal.softwareheritage.org:9200"
@@ -131,6 +142,7 @@ def _get_save_origin_task_info_test(
             "started": task_run["started"],
             "ended": task_run["ended"],
             "status": task_run["status"],
+            "visit_status": sor.visit_status,
         }
         if not task_archived
         else {}
@@ -229,11 +241,16 @@ def test_get_save_origin_requests_find_visit_date(mocker):
     mock_get_origin_visits.assert_called_once()
 
 
-def _get_save_origin_requests(mocker, load_status, visit_status):
-    # create a save request
+def _get_save_origin_requests(
+    mocker, load_status, visit_status, request_date: Optional[datetime] = None
+):
+    """Wrapper around the get_origin_save_origin_request call.
+
+    """
     SaveOriginRequest.objects.create(
         request_date=datetime.now(tz=timezone.utc),
         visit_type=_visit_type,
+        visit_status=visit_status,
         origin_url=_origin_url,
         status=SAVE_REQUEST_ACCEPTED,
         visit_date=None,
@@ -256,7 +273,7 @@ def _get_save_origin_requests(mocker, load_status, visit_status):
         formatted_date="",
         metadata={},
         origin=_origin_url,
-        snapshot=None,
+        snapshot="",  # make mypy happy
         status=visit_status,
         type=_visit_type,
         url="",
@@ -270,35 +287,206 @@ def _get_save_origin_requests(mocker, load_status, visit_status):
     return sors
 
 
+@pytest.mark.parametrize("visit_date", [None, "some-date"])
+def test_from_save_origin_request_to_save_request_info_dict(visit_date):
+    """Ensure save request to json serializable dict is fine
+
+    """
+    request_date = datetime.now(tz=timezone.utc)
+    _visit_date = request_date + timedelta(minutes=5) if visit_date else None
+    request_date = datetime.now(tz=timezone.utc)
+    sor = SaveOriginRequest(
+        request_date=request_date,
+        visit_type=_visit_type,
+        visit_status=VISIT_STATUS_FULL,
+        origin_url=_origin_url,
+        status=SAVE_REQUEST_ACCEPTED,
+        loading_task_status=None,
+        visit_date=_visit_date,
+        loading_task_id=1,
+    )
+
+    assert sor.to_dict() == SaveOriginRequestInfo(
+        id=sor.id,
+        origin_url=sor.origin_url,
+        visit_type=sor.visit_type,
+        save_request_date=sor.request_date.isoformat(),
+        save_request_status=sor.status,
+        save_task_status=sor.loading_task_status,
+        visit_status=sor.visit_status,
+        visit_date=_visit_date.isoformat() if _visit_date else None,
+        loading_task_id=sor.loading_task_id,
+    )
+
+
+def test__check_origin_exists_404(requests_mock):
+    url_ko = "https://example.org/some-inexistant-url"
+    requests_mock.head(url_ko, status_code=404)
+
+    with pytest.raises(BadInputExc, match="not exist"):
+        _check_origin_exists(url_ko)
+
+
+def test__check_origin_exists_200(requests_mock):
+    url = "https://example.org/url"
+    requests_mock.head(url, status_code=200)
+
+    # passes the check
+    _check_origin_exists(url)
+
+
+def test_origin_exists_404(requests_mock):
+    """Origin which does not exist should be reported as inexistent"""
+    url_ko = "https://example.org/some-inexistant-url"
+    requests_mock.head(url_ko, status_code=404)
+
+    actual_result = origin_exists(url_ko)
+    assert actual_result == OriginExistenceCheckInfo(
+        origin_url=url_ko, exists=False, last_modified=None, content_length=None,
+    )
+
+
+def test_origin_exists_200_no_data(requests_mock):
+    """Existing origin should be reported as such (no extra information)"""
+    url = "http://example.org/real-url"
+    requests_mock.head(
+        url, status_code=200,
+    )
+
+    actual_result = origin_exists(url)
+    assert actual_result == OriginExistenceCheckInfo(
+        origin_url=url, exists=True, last_modified=None, content_length=None,
+    )
+
+
+def test_origin_exists_200_with_data(requests_mock):
+    """Existing origin should be reported as such (+ extra information)"""
+    url = "http://example.org/real-url"
+    requests_mock.head(
+        url,
+        status_code=200,
+        headers={
+            "content-length": "10",
+            "last-modified": "Sun, 21 Aug 2011 16:26:32 GMT",
+        },
+    )
+
+    actual_result = origin_exists(url)
+    assert actual_result == OriginExistenceCheckInfo(
+        origin_url=url,
+        exists=True,
+        content_length=10,
+        last_modified="Sun, 21 Aug 2011 16:26:32 GMT",
+    )
+
+
 @pytest.mark.django_db
-def test_get_save_origin_requests_no_visit_date_found(mocker):
+@pytest.mark.parametrize("visit_status", ["created", "ongoing",])
+def test_get_save_origin_requests_no_visit_date_found(mocker, visit_status):
+    """Uneventful visits with failed visit status are marked as failed
+
+    """
     sors = _get_save_origin_requests(
-        mocker, load_status="scheduled", visit_status="created"
+        mocker, load_status="scheduled", visit_status=visit_status,
     )
     # check no visit date has been found
     assert len(sors) == 1
     assert sors[0]["save_task_status"] == SAVE_TASK_RUNNING
     assert sors[0]["visit_date"] is None
+    assert sors[0]["visit_status"] == visit_status
 
 
 @pytest.mark.django_db
-def test_get_save_origin_requests_failed_when_not_found(mocker):
+@pytest.mark.parametrize("visit_status", ["not_found", "failed",])
+def test_get_save_origin_requests_no_failed_status_override(mocker, visit_status):
+    """Uneventful visits with failed statuses (failed, not found) are marked as failed
+
+    """
     sors = _get_save_origin_requests(
-        mocker, load_status="uneventful", visit_status="not_found"
+        mocker, load_status="uneventful", visit_status=visit_status
     )
     assert len(sors) == 1
-    assert sors[0]["save_task_status"] == SAVE_TASK_FAILED
-    assert sors[0]["visit_date"] is not None
 
-
-@pytest.mark.django_db
-def test_get_save_origin_requests_no_failed_status_override(mocker):
-    sors = _get_save_origin_requests(
-        mocker, load_status="uneventful", visit_status="not_found"
-    )
-    assert len(sors) == 1
     assert sors[0]["save_task_status"] == SAVE_TASK_FAILED
-    assert sors[0]["visit_date"] is not None
+    visit_date = sors[0]["visit_date"]
+    if visit_status == "failed":
+        assert visit_date is None
+    else:
+        assert visit_date is not None
 
     sors = get_save_origin_requests(_visit_type, _origin_url)
+    assert len(sors) == 1
     assert sors[0]["save_task_status"] == SAVE_TASK_FAILED
+    assert sors[0]["visit_status"] == visit_status
+
+
+@pytest.mark.django_db
+@pytest.mark.parametrize(
+    "load_status,visit_status",
+    [("eventful", "full"), ("eventful", "partial"), ("uneventful", "partial"),],
+)
+def test_get_visit_info_for_save_request_succeeded(mocker, load_status, visit_status):
+    """Nominal scenario, below 30 days, returns something"""
+    sors = _get_save_origin_requests(
+        mocker, load_status=load_status, visit_status=visit_status
+    )
+    assert len(sors) == 1
+
+    assert sors[0]["save_task_status"] == SAVE_TASK_SUCCEEDED
+    assert sors[0]["visit_date"] is not None
+    assert sors[0]["visit_status"] == visit_status
+
+    sors = get_save_origin_requests(_visit_type, _origin_url)
+    assert sors[0]["save_task_status"] == SAVE_TASK_SUCCEEDED
+    assert sors[0]["visit_status"] == visit_status
+
+
+@pytest.mark.django_db
+@pytest.mark.parametrize("load_status", ["eventful", "uneventful",])
+def test_get_visit_info_incomplete_visit_still_successful(mocker, load_status):
+    """Incomplete visit information, yet the task is considered ok
+
+    """
+    sors = _get_save_origin_requests(
+        mocker, load_status=load_status, visit_status=None,
+    )
+    assert len(sors) == 1
+
+    assert sors[0]["save_task_status"] == SAVE_TASK_SUCCEEDED
+    assert sors[0]["visit_date"] is None
+    assert sors[0]["visit_status"] is None
+
+    # nothing to refresh so nothing to return
+    assert len(refresh_save_origin_request_statuses()) == 0
+
+
+@pytest.mark.django_db
+def test_refresh_save_request_statuses(mocker, api_client):
+    """Refresh filters non-terminal save origins requests and update if changes
+
+    """
+    sors = _get_save_origin_requests(
+        mocker, load_status=SAVE_TASK_SCHEDULED, visit_status=None,
+    )
+    assert len(sors) == 1
+
+    # no changes so refresh does detect the entry but does nothing
+    sors = refresh_save_origin_request_statuses()
+
+    assert len(sors) == 1
+    for sor in sors:
+        # as it turns out, in this test, this won't update anything as no new status got
+        # returned by the scheduler
+        assert sor["save_task_status"] == SAVE_TASK_SCHEDULED
+
+    # make the scheduler return eventful for that task
+    _mock_scheduler(mocker)
+
+    # Detected entry, this time it should be updated
+    sors = refresh_save_origin_request_statuses()
+
+    assert len(sors) == 1
+    for sor in sors:
+        # as it turns out, in this test, this won't update anything as no new status got
+        # returned by the scheduler
+        assert sor["save_task_status"] == SAVE_TASK_SUCCEEDED
